/*
 * TillPositionListenerProxy
 *
 * 07/074/04
 *
 * Copyright:
 * Licensed Materials - Property of IBM
 * "Restricted Materials of IBM"
 * 5724-AEF
 * (C) Copyright IBM Corp. 2003.
 *
 */
package com.userco.retail.AEF.client;

import com.ibm.retail.AEF.event.*;
import com.ibm.retail.AEF.data.*;
import com.ibm.retail.AEF.client.*;
import com.ibm.retail.AEF.util.*;
import com.ibm.retail.si.util.*;
import com.ibm.retail.si.Copyright;

import com.userco.retail.AEF.event.*;

import java.rmi.*;
import java.rmi.server.*;
import java.lang.reflect.*;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

/**
 * The TillPositionListenerProxy provides a proxy object for remote listeners 
 * to monitor the TillPosition events of its associated terminal session.  
 * <p>
 * Remote proxy objects extend the appropriate RMI server classes which handle the   
 * RMI duties for the client (so the client does not need to deal with rmi server issues). 
 * The proxy objects also perform the role of event dispatching.
 * <p>
 * By default, the listener proxy object will perform event queuing on an AEF event thread.
 * This relieves the client of any "thread swapping" responsibilities and insures that the
 * AEF event dispatching performance is not affected by client processing.  To override this
 * default behavior, use the <i>setDispatchQueue()</i> method.  The proxy utilizes the 
 * EventDispatcher for queueing event listener notification.
 * <p>
 * To use a proxy object, the client must implement the listener interface
 * and get the POSDataProvider object from the AEFSession to monitor.  The proxy
 * object is then constructed passing a reference to the client and the data provider. 
 * The proxy object performs the listener registration and forwards
 * all events from the data provider to the client.
 *
 */

public class TillPositionListenerProxy extends AEFEventListenerProxy implements AEFListenerProxyInterface, 
                                                                                TillPositionListener
{
   static String copyright()
   { return com.ibm.retail.si.Copyright.IBM_COPYRIGHT_SHORT;}

   private static Log log = LogFactory.getLog(TillPositionListenerProxy.class);

   protected static Method listenerMethod = getListenerMethod(TillPositionEvent.class,TillPositionListener.class,"tillPositionChanged"); 

   /**
    * Construct an TillPositionListenerProxy.
    * This method registers the proxy as a listener with the POSDataProvider and begins event
    * notification to the proxy client listener.
    * 
    * @param dataProvider instance of POSDataProvider which is the source of events 
    * @param listener client listener which receives the event notification via this proxy
    * 
    * @exception java.rmi.RemoteException
    * @exception AEFException 
    *    Among the possible AEFException error codes are:
    *       <br>AEFConst.NO_LISTENER_SUPPORT
    *
    */
   public TillPositionListenerProxy(POSDataProvider dataProvider, TillPositionListener listener) throws RemoteException, AEFException
   {
      this.dataProvider = dataProvider;
      this.listener     = listener;
      dataProvider.addPOSAppEventListener(TillPositionListener.class.getName(),this);
   }

   /**
    * Remove the proxy listener from the POSDataProvider. Ends notification of events.
    *
    * @exception java.rmi.RemoteException
    * @exception AEFException 
    *    Among the possible AEFException error codes are:
    *       <br>AEFConst.NO_LISTENER_SUPPORT
    *
    */
   public void removeListener() throws RemoteException, AEFException
   {
      dataProvider.removePOSAppEventListener(TillPositionListener.class.getName(),this);
   }

   /**
    * Till position changed on the POS terminal session.
    * Dispatches event notification to the proxy client listener.
    *
    * @param   evt 
    * @throws  RemoteException if remote access fails
    *
    */
   public void tillPositionChanged(TillPositionEvent evt) throws RemoteException
   {
      dispatchEvent(evt,listener,listenerMethod);
   }

   // instance data
   protected POSDataProvider        dataProvider;
   protected TillPositionListener       listener;
}
