/*
 * ButtonPalette
 *
 * 07/07/04
 *
 * Copyright:
 * Licensed Materials - Property of IBM
 * "Restricted Materials of IBM"
 * 5724-AEF
 * (C) Copyright IBM Corp. 2004.
 *
 */
package com.userco.retail.client.gui;

import java.awt.*;
import java.awt.event.*;
import javax.swing.*;
import javax.swing.border.*;

import java.util.logging.*;

import com.ibm.retail.AEF.data.*;
import com.ibm.retail.AEF.session.*;
import com.ibm.retail.AEF.event.*;
import com.ibm.retail.AEF.client.*;

import com.ibm.retail.si.Copyright;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

/**
 * This class is used to display a palette of action buttons such as 
 * "logon", "logoff", "start trans", etc.
 */
public class ButtonPalette extends JPanel
{

   static String copyright()
      { return com.ibm.retail.si.Copyright.IBM_COPYRIGHT_SHORT; }

   private static Log log = LogFactory.getLog(ButtonPalette.class);


   /**
    * The constructor
    *
    * @param session The session which will provide the operator display
    *                data.
    * @param frame   The main SampleGUI window
    * @param controller The session controller
    */
   public ButtonPalette(AEFSession session, 
                        Frame      frame,
                        Controller controller)
   {      
      super();

      this.session = session;
      this.frame = frame;
      this.controller = controller;

      // Get the data provider.
      try
      {
         dataProvider = session.getPOSDataProvider();   
      }
      catch (Throwable th)
      {
         log.error("Error getting POSDataProvider from session.", th);
      }



      Border raisedBevel = BorderFactory.createRaisedBevelBorder();
      Border loweredBevel = BorderFactory.createLoweredBevelBorder();



      setBorder(BorderFactory.createCompoundBorder(
                raisedBevel, loweredBevel));
      setLayout(new GridLayout(7,2));

      logonButton = new JButton();
      logonAction = new LogonAction();
      logonAction.initialize("Logon",
                             session,
                             frame,
                             controller,
                             "Display logon dialog.",
                             (Integer)null,
                             (Icon)null,
                             "L");
      logonButton.setAction(logonAction);
      add(logonButton);
      logoffButton = new JButton();
      logoffAction = new LogoffAction();
      logoffAction.initialize("Logoff",
                             session,
                             frame,
                             controller,
                             "Log off the session.",
                             (Integer)null,
                             (Icon)null,
                             "O");
      logoffButton.setAction(logoffAction);
      add(logoffButton);
      startTranButton = new JButton();
      startTranAction = new StartTransAction();
      startTranAction.initialize("Start Trans.",
                             session,
                             frame,
                             controller,
                             "Start a sales transaction.",
                             (Integer)null,
                             (Icon)null,
                             "S");
      startTranButton.setAction(startTranAction);
      add(startTranButton);
      voidTranButton = new JButton();
      voidTranAction = new VoidTransAction();
      voidTranAction.initialize("Void Trans.",
                             session,
                             frame,
                             controller,
                             "Voids the current transaction.",
                             (Integer)null,
                             (Icon)null,
                             "V");
      voidTranButton.setAction(voidTranAction);
      add(voidTranButton);
      addItemButton = new JButton();
      addItemAction = new AddItemAction();
      addItemAction.initialize("Add Item",
                             session,
                             frame,
                             controller,
                             "Add an item to the transaction.",
                             (Integer)null,
                             (Icon)null,
                             "A");
      addItemButton.setAction(addItemAction);
      add(addItemButton);
      voidItemButton = new JButton();
      voidItemAction = new VoidItemAction();
      voidItemAction.initialize("Void Item",
                             session,
                             frame,
                             controller,
                             "Remove an item from the transaction.",
                             (Integer)null,
                             (Icon)null,
                             "R");
      voidItemButton.setAction(voidItemAction);
      add(voidItemButton);
      addCreditButton = new JButton();
      creditTenderAction = new CreditTenderAction();
      creditTenderAction.initialize("Credit Tender",
                             session,
                             frame,
                             controller,
                             "Add a credit card tender to the transaction.",
                             (Integer)null,
                             (Icon)null,
                             "C");
      addCreditButton.setAction(creditTenderAction);
      add(addCreditButton);
      keySeqButton = new JButton();
      keySequenceAction = new KeySequenceAction();
      keySequenceAction.initialize("Key Sequence",
                             session,
                             frame,
                             controller,
                             "Send a key sequence directly to the POS application.",
                             (Integer)null,
                             (Icon)null,
                             "K");
      keySeqButton.setAction(keySequenceAction);
      add(keySeqButton);
      exitButton = new JButton();
      exitAction = new ExitAction();
      exitAction.initialize("Exit",
                            session,
                            frame,
                            controller,
                            "Exit the program.",
                            (Integer)null,
                            (Icon)null,
                            "E");
      exitButton.setAction(exitAction);
      add(exitButton);


      // Get the initial states from the data provider.
      if (dataProvider != null)
      {
         Object value = null;
         try
         {
            value = dataProvider.getPropertyValue(WorkstationStatusProperties.CATEGORY,
                                                  WorkstationStatusProperties.SIGNON_STATUS); 

            if (value != null)
            {
               if (value.equals("true"))
               {
                  // Operator is signed on, disable the logon action.
                  logonAction.setEnabled(false);
               }
               else if (value.equals("false"))
               {
                  // Operator is signed off, disable the logoff action.
                  logoffAction.setEnabled(false);
               }
               else if (value.equals("secureMode"))
               {
                  // Operator is in secure mode, disable the logoff action.
                  logoffAction.setEnabled(false);
               }
            }

         }
         catch (Throwable th2)
         {
            log.error("Unable to fetch initial button palette status.", th2);
         }
      }



      // Create a proxy for listening to property change events
      // from POSDataProvider.
      try
      {
         proxy = new AEFPropertyListenerProxyImpl(dataProvider,
                                                  new StatusPropertyChangeListener());
      }
      catch (Throwable th3)
      {
         log.error("Unable to attach a status property change listener.", th3);
      }
   }

   /**
    * Causes any components interested in data provider events
    * to start listening.
    *
    */
   public void startListening()
   {      
      try
      {
         proxy.addAEFPropertyChangeListener(WorkstationStatusProperties.CATEGORY,
                                            WorkstationStatusProperties.SIGNON_STATUS);
      }
      catch (Throwable th)
      {
         log.error("ButtonPalette unable to start listening.", th);
      }
   }

   /**
    * Causes any components interested in data provider events
    * to stop listening.
    *
    */
   public void stopListening()
   {      
      try
      {
         proxy.removeAEFPropertyChangeListener(WorkstationStatusProperties.CATEGORY,
                                               WorkstationStatusProperties.SIGNON_STATUS);
      }
      catch (Throwable th)
      {
         log.error("ButtonPalette unable to stop listening.", th);
      }
   }


  /**
   * Inner AEFPropertyChangeListener class which listens for
   * property changes to update the buttons.
   *
   */
   class StatusPropertyChangeListener implements AEFPropertyChangeListener
   {
      /**
       * An AEF property has changed.
       *
       * @param  evt contains details of the event
       * @throws RemoteException if remote access fails
       *
       */
      public void propertyChanged(AEFPropertyChangeEvent evt)
      {
         if (evt.getCategoryName().equals(WorkstationStatusProperties.CATEGORY))
         {
            if (evt.getPropertyName().equals(WorkstationStatusProperties.SIGNON_STATUS))
            {
               if (evt.getNewValue().toString().equals("false"))
               {
                  // Operator is no longer signed on, enable the logon action,
                  // disable the logoff action.
                  logonAction.setEnabled(true);
                  logoffAction.setEnabled(false);
               }
               else if (evt.getNewValue().toString().equals("true"))
               {
                  // Operator is now signed on, disable the logon action,
                  // enable the logoff action.
                  logonAction.setEnabled(false);
                  logoffAction.setEnabled(true);
               }
               else if (evt.getNewValue().toString().equals("secureMode"))
               {
                  // Operator is now in secure mode, and can only sign back on.
                  // Enable the logon action, disable the logoff action.
                  logonAction.setEnabled(true);
                  logoffAction.setEnabled(false);
               }
            }
         }
      }
   }



   // Member variables
   protected JButton                        logonButton;
   protected LogonAction                    logonAction;
   protected LogoffAction                   logoffAction;
   protected StartTransAction               startTranAction;
   protected VoidTransAction                voidTranAction;
   protected AddItemAction                  addItemAction;
   protected VoidItemAction                 voidItemAction;
   protected CreditTenderAction             creditTenderAction;
   protected KeySequenceAction              keySequenceAction;
   protected ExitAction                     exitAction;
   protected JButton                        logoffButton;
   protected JButton                        startTranButton;
   protected JButton                        voidTranButton;
   protected JButton                        addItemButton;
   protected JButton                        voidItemButton;
   protected JButton                        addCreditButton;
   protected JButton                        keySeqButton;
   protected JButton                        exitButton;
   protected POSDataProvider                dataProvider;
   protected AEFPropertyListenerProxy       proxy;
   protected AEFSession                     session;
   protected Frame                          frame;
   protected Controller                     controller;

}
