/*
 * Controller
 *
 * 07/07/04
 *
 * Copyright:
 * Licensed Materials - Property of IBM
 * "Restricted Materials of IBM"
 * 5724-AEF
 * (C) Copyright IBM Corp. 2004.
 *
 */
package com.userco.retail.client.gui;

import com.ibm.retail.AEF.session.*;
import com.ibm.retail.AEF.automation.*;
import com.ibm.retail.AEF.data.*;
import com.ibm.retail.AEF.workstation.*;
import java.util.logging.*;
import java.util.*;
import java.lang.ref.*;

import com.ibm.retail.si.Copyright;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

/**
 * This class is used as the controller interface to the AEF Automation API.
 * All SampleGUI calls to the automation API go through the controller.
 */
public class Controller 
{

   static String copyright()
      { return com.ibm.retail.si.Copyright.IBM_COPYRIGHT_SHORT; }

   private static Log log = LogFactory.getLog(Controller.class);




   /**
    * Default constructor.
    *
    * @param boolean True if the requested session was a specific
    *                terminal number.  False if the session was 
    *                any available session.
    */
   public Controller(boolean specificSession)
   {      
      specific = specificSession;
   }

   /**
    * Sets the session.
    *
    * @param session
    */
   public void setSession(AEFSession session)
   {      
      this.session = session;
      try
      {
         automation = session.getPOSAutomationProvider();
      }
      catch (Throwable th)
      {
         log.error("Unable to retrieve POSAutomationProvider.",th);
      }
   }


   /**
    * Gets the session.
    *
    * @return AEFSession
    */
   public AEFSession getSession()
   {      
      return session;
   }


   /**
    * Gets the transaction if one is in progress, otherwise returns null.
    *
    * @return Transaction
    */
   public Transaction getTransaction()
   {      
      Transaction retVal = null;
      if (automation == null)
      {
         log.error("Cannot get transaction because POSAutomationProvider API is unavailable.");
      }
      else
      {
         try
         {
            retVal = automation.getTransaction();
         }
         catch (Throwable th)
         {
            log.error("Unable to retrieve transaction from POSAutomationProvider.", th);
         }
      }
      return retVal;
   }


   /**
    * Logs onto the session with the OperatorIdentifer arguments.
    *
    * @return Operator
    */
   public Operator logon(OperatorIdentifier opIdent)
   {      
      Operator retVal = null;

      if (automation == null)
      {
         log.error("Cannot logon because POSAutomationProvider API not available.");
      }
      else
      {
         try
         {
            retVal = automation.logon(opIdent);
         }
         catch (Throwable th)
         {
            // Perform error handling here.
            log.error("Unable to logon.", th);
         }
      }
      return retVal;
   }


   /**
    * Logs off the session.
    *
    */
   public void logoff()
   {      
      if (automation == null)
      {
         log.error("Cannot logoff because POSAutomationProvider API not available.");
      }
      else
      {
         try
         {
            automation.logoff();
         }
         catch (Throwable th)
         {
            // Perform error handling here.
            log.error("Unable to logoff.", th);
         }
      }
   }



   /**
    * Starts a regular sales transaction.
    *
    * @return SalesTransaction
    */
   public SalesTransaction startTransaction(TransactionIdentifier transIdent)
   {      
      SalesTransaction retVal = null;

      if (automation == null)
      {
         log.error("Cannot start transaction because POSAutomationProvider API not available.");
      }
      else
      {
         try
         {
            retVal = (SalesTransaction)(automation.startTransaction(transIdent));
         }
         catch (Throwable th)
         {
            // Perform error handling here.
            log.error("Unable to start transaction.", th);
         }
      }
      return retVal;
   }


   /**
    * Voids the transaction in progress.
    *
    */
   public void voidTransaction()
   {      
      if (automation == null)
      {
         log.error("Cannot void transaction because POSAutomationProvider API not available.");
      }
      else
      {
         try
         {
            automation.voidCurrentTransaction();
         }
         catch (Throwable th)
         {
            // Perform error handling here.
            log.error("Unable to void transaction.", th);
         }
      }
   }

   /**
    * Releases the session.
    *
    */
   public void releaseSession()
   {      
      if (!specific)
      {
         // We only need to release the session if it was gotten
         // via a call to getAvailableSession.
         try
         {
            session.release();
         }
         catch (Throwable th)
         {
            // Perform error handling here.
            log.error("Unable to release the session.", th);
         }
      }
   }



   /**
    * Adds one or more items to the sales transaction.
    *
    * @param itemIdent The ItemIdentifier containing arguments such 
    *                  as the itemcode, quantity, etc.
    * @return ArrayList of LineItems.
    */
   public ArrayList addItem(ItemIdentifier itemIdent)
   {      
      ArrayList retVal = null;

      if (automation == null)
      {
         log.error("Cannot add items because POSAutomationProvider API not available.");
      }
      else
      {
         SalesTransaction trans = (SalesTransaction)(getTransaction());
         if (trans == null)
         {
            log.error("Cannot add items because a transaction is not in progress.");
         }
         else
         {
            try
            {
               retVal = trans.addItem(itemIdent);
            }
            catch (Throwable th)
            {
               // Perform error handling here.
               log.error("Unable to add item.", th);
            }
         }
      }
      return retVal;
   }




   /**
    * Voids an item from the sales transaction.
    *
    * @param itemIdent The ItemIdentifier containing arguments such 
    *                  as the itemcode, quantity, etc.
    */
   public void voidItem(ItemIdentifier itemIdent)
   {      

      if (automation == null)
      {
         log.error("Cannot add items because POSAutomationProvider API not available.");
      }
      else
      {
         SalesTransaction trans = (SalesTransaction)(getTransaction());
         if (trans == null)
         {
            log.error("Cannot void item because a transaction is not in progress.");
         }
         else
         {
            try
            {
               trans.voidItem(itemIdent);
            }
            catch (Throwable th)
            {
               // Perform error handling here.
               log.error("Unable to void item.", th);
            }
         }
      }
   }




   /**
    * Adds a tender to the sales transaction.
    *
    * @param creditIdent The MSRCreditIdentifier
    *                    containing arguments such 
    *                    as the tender amount, card type and track data.
    * @return ArrayList of LineItems.
    */
   public ArrayList addTender(MSRCreditIdentifier creditIdent)
   {      
      ArrayList retVal = null;

      if (automation == null)
      {
         log.error("Cannot add tender because POSAutomationProvider API not available.");
      }
      else
      {
         SalesTransaction trans = (SalesTransaction)(getTransaction());
         if (trans == null)
         {
            log.error("Cannot add items because a transaction is not in progress.");
         }
         else
         {
            try
            {
               retVal = trans.addTender(creditIdent);
            }
            catch (Throwable th)
            {
               // Perform error handling here.
               log.error("Unable to add tender.", th);
            }
         }
      }
      return retVal;
   }




   /**
    * Sends a key sequence to the POS application.
    *
    * @param sequence    The key sequence to be sent.  Function codes must
    *                    be inside < > characters.
    */
   public void keySequence(String sequence)
   {      
      Workstation workstation = null;


      try
      {
         workstation = session.getWorkstation();
      }
      catch (Throwable th)
      {
         log.error("Unable to retrieve Workstation to send the key sequence.", th);
         return;
      }

      try
      {
         workstation.sendKeySequence(sequence);
      }
      catch (Throwable th2)
      {
         log.error("Unable to send key sequence to POS application.", th2);
      }

   }



   // Member variables
   protected AEFSession            session;
   protected POSAutomationProvider automation;
   protected boolean               specific;
}
